#define testtree_cxx
// The class definition in testtree.h has been generated automatically
// by the ROOT utility TTree::MakeSelector(). This class is derived
// from the ROOT class TSelector. For more information on the TSelector
// framework see $ROOTSYS/README/README.SELECTOR or the ROOT User Manual.

// The following methods are defined in this file:
//    Begin():        called every time a loop on the tree starts,
//                    a convenient place to create your histograms.
//    SlaveBegin():   called after Begin(), when on PROOF called only on the
//                    slave servers.
//    Process():      called for each event, in this function you decide what
//                    to read and fill your histograms.
//    SlaveTerminate: called at the end of the loop on the tree, when on PROOF
//                    called only on the slave servers.
//    Terminate():    called at the end of the loop on the tree,
//                    a convenient place to draw/fit your histograms.
//
// To use this file, try the following session on your Tree T:
//
// Root > T->Process("testtree.C")
// Root > T->Process("testtree.C","some options")
// Root > T->Process("testtree.C+")
//

#include "testtree.h"
#include <TH2.h>
#include <TStyle.h>
#include "TMath.h"
#include <iostream>

void testtree::Begin(TTree * /*tree*/)
{
   // The Begin() function is called at the start of the query.
   // When running with PROOF Begin() is only called on the client.
   // The tree argument is deprecated (on PROOF 0 is passed).

   TString option = GetOption();

}

void testtree::SlaveBegin(TTree * /*tree*/)
{
   // The SlaveBegin() function is called after the Begin() function.
   // When running with PROOF SlaveBegin() is called on each slave server.
   // The tree argument is deprecated (on PROOF 0 is passed).

   TString option = GetOption();
   fProofFile = new TProofOutputFile("out_0.root");
   fProofFile->SetOutputFileName("out_0.root");
   // This is the info we send to the master
   fOutput->Add(fProofFile);
   TDirectory *savedir = gDirectory;
   if (!(outputfile = fProofFile->OpenFile("RECREATE"))) {
     Warning("SlaveBegin", "problems opening file: %s/%s",
	     fProofFile->GetDir(), fProofFile->GetFileName());
   }
   savedir->cd();
   outputtree = new TTree("test0","test0");
   outputtree->Branch("par",&par,"par/D");
   outputtree->SetDirectory(outputfile);
   outputtree->AutoSave();
   listoftreesandfiles.push_back(std::pair<TFile*,TTree*>(outputfile,
							  outputtree));
   fProofFile = new TProofOutputFile("out_1.root");
   fProofFile->SetOutputFileName("out_1.root");
   // This is the info we send to the master
   fOutput->Add(fProofFile);
   if (!(outputfile = fProofFile->OpenFile("RECREATE"))) {
     Warning("SlaveBegin", "problems opening file: %s/%s",
	     fProofFile->GetDir(), fProofFile->GetFileName());
   }
   savedir->cd();
   outputtree = new TTree("test1","test1");
   outputtree->Branch("par",&par,"par/D");
   outputtree->SetDirectory(outputfile);
   outputtree->AutoSave();
   listoftreesandfiles.push_back(std::pair<TFile*,TTree*>(outputfile,
							  outputtree));
   fProofFile = new TProofOutputFile("out_2.root");
   fProofFile->SetOutputFileName("out_2.root");
   // This is the info we send to the master
   fOutput->Add(fProofFile);
   if (!(outputfile = fProofFile->OpenFile("RECREATE"))) {
     Warning("SlaveBegin", "problems opening file: %s/%s",
	     fProofFile->GetDir(), fProofFile->GetFileName());
   }
   savedir->cd();
   outputtree = new TTree("test2","test2");
   outputtree->Branch("par",&par,"par/D");
   outputtree->SetDirectory(outputfile);
   outputtree->AutoSave();
   listoftreesandfiles.push_back(std::pair<TFile*,TTree*>(outputfile,
							  outputtree));
   fProofFile = new TProofOutputFile("out_3.root");
   fProofFile->SetOutputFileName("out_3.root");
   // This is the info we send to the master
   fOutput->Add(fProofFile);
   if (!(outputfile = fProofFile->OpenFile("RECREATE"))) {
     Warning("SlaveBegin", "problems opening file: %s/%s",
	     fProofFile->GetDir(), fProofFile->GetFileName());
   }
   savedir->cd();
   outputtree = new TTree("test3","test3");
   outputtree->Branch("par",&par,"par/D");
   outputtree->SetDirectory(outputfile);
   outputtree->AutoSave();
   listoftreesandfiles.push_back(std::pair<TFile*,TTree*>(outputfile,
							  outputtree));
   gDirectory = savedir;
}

Bool_t testtree::Process(Long64_t entry)
{
   // The Process() function is called for each entry in the tree (or possibly
   // keyed object in the case of PROOF) to be processed. The entry argument
   // specifies which entry in the currently loaded tree is to be processed.
   // It can be passed to either testtree::GetEntry() or TBranch::GetEntry()
   // to read either all or the required parts of the data. When processing
   // keyed objects with PROOF, the object is already loaded and is available
   // via the fObject pointer.
   //
   // This function should contain the "body" of the analysis. It can contain
   // simple or elaborate selection criteria, run algorithms on the data
   // of the event and typically fill histograms.
   //
   // The processing can be stopped by calling Abort().
   //
   // Use fStatus to set the return value of TTree::Process().
   //
   // The return value is currently not used.
  GetEntry(entry);
  Int_t region = (Int_t)TMath::Floor(fabs(par));
  if(region>3) region = 3;
  switch(region){
  case 0:{
    listoftreesandfiles[0].second->Fill();
  }break;
  case 1:{
    listoftreesandfiles[1].second->Fill();
  }break;
  case 2:{
    listoftreesandfiles[2].second->Fill();
  }break;
  case 3:{
    listoftreesandfiles[3].second->Fill();
  }break;
  default:
    break;
  };
  return kTRUE;
}

void testtree::SlaveTerminate()
{
   // The SlaveTerminate() function is called after all entries or objects
   // have been processed. When running with PROOF SlaveTerminate() is called
   // on each slave server.
  TDirectory *savedir = gDirectory;
  std::vector<std::pair<TFile*,TTree*> >::iterator trees =
    listoftreesandfiles.begin();
  for(;trees!=listoftreesandfiles.end(); trees++){
    outputfile = trees->first;
    outputfile->cd();
    trees->second->Write();
    trees->second->SetDirectory(0);
    outputfile->Close();
  }
  gDirectory = savedir;
}

void testtree::Terminate()
{
   // The Terminate() function is the last function to be called during
   // a query. It always runs on the client, it can be used to present
   // the results graphically or save the results to file.

}
